/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.testsuites;

import java.io.IOException;
import java.util.List;



import org.thermostat.qa.framework.Assert;
import org.thermostat.qa.framework.ThermostatTest;

/**
 * Class AgentTest
 * contains basic smoke tests for starting the Thermostat agent. Starting the agent without
 * storage (which should not work) and with storage is checked.
 *
 */
public class AgentTest extends ThermostatTest
{
    private static final String[] THERMOSTAT_AGENT_WITHOUT_STORAGE_MESSAGE = new String[] {
        "list of commands:",
        ""
        };
    
    @Override
    protected void tearDown()
    {
        // TODO Auto-generated method stub
        
    }

    @Override
    protected void setUp()
    {
        // TODO Auto-generated method stub
        
    }

    /**
     * Method testStartWithoutStorage
     * tries to start thermostat agent in new thread without previously
     * starting the storage. There should be no thermostat agents running.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     */
    public void testStartWithoutStorage() throws IOException
    {
        //start only agent
        new AgentThread().start();
        logInfo("runAgentThread", "Thermostat Agent started");
        
        //wait till agent stops
        sleep(FIVE_SEC);
        
        //there should not be any agent running (check the jps)
        List<String> output = runHelperBashScript("list_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");
        for (String message : output)
        {
            logInfo("Agent list:", message);
        }
        Assert.assertTrue(output.size()==0, "there should not be any running thermostat agents");
    }
    
    /**
     * Method testStartAgent
     * starts the storage and then tries to start thermostat agent in a new
     * thread. There should be one thermostat agent running.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     */
    public void testStartAgent() throws IOException
    {
        startStorage();
        
        //start agent
        new AgentThread().start();
        logInfo("runAgentThread", "Thermostat Agent started");
        
        //check if the agent is running (jps)
        List<String> output = runHelperBashScript("list_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");
        Assert.assertTrue(output.size()>0, "there should be >0 running thermostat agents");
                
        //stop the agent
        output = runHelperBashScript("stop_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'stop_all_therm_agents.sh'");
        
        stopStorage();
    }
    
    public static void main(String[] args) {
        new AgentTest().doTests(args);
    }

}
